@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -14,6 +14,7 @@
 #include "queue.h"
 #include "dlist.h"
 #include "sort.h"
+#include "mutex.h"
 
 class tDownloadTree:public tAbstractSortTree{
 	protected:
@@ -31,25 +32,26 @@ struct tStringHostNode:public tAbstractSortNode{
 };
 
 class tHostTree:public tAbstractSortTree{
-	protected:
-	public:
-		tStringHostNode *find(char *what,int port);
+protected:
+public:
+	tStringHostNode *find(const char *what,int port);
+	~tHostTree();
 };
 
 class tDB{
 	tHostTree *tree;
 	tDownloadTree **hash(tStringHostNode *temp,tDownload *what);
-	pthread_mutex_t mylock;
-	public:
-		tDB();
-		void lock();
-		void unlock();
-		void insert(tDownload *what);
-		void del(tDownload *what);
-		int empty();
-		tDownload *find(tDownload *what);
-		tDownload *find(tAddr *addr);
-		~tDB();
+public:
+	d4xMutex mylock;
+	tDB();
+	void insert(tDownload *what);
+	void del(tDownload *what);
+	int empty();
+	void lock(){mylock.lock();};
+	void unlock(){mylock.unlock();};
+	tDownload *find(tDownload *what);
+	tDownload *find(const d4x::URL &addr);
+	~tDB();
 };
 
 #endif
