@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -15,23 +15,34 @@
 #include <pthread.h>
 #include "../dlist.h"
 #include "../main.h"
-#include "limface.h"
 #include "lod.h"
 #include "graph.h"
-
-extern tMain aa;
+#include "qtree.h"
 
 extern GtkWidget *MainMenu;
-extern GtkWidget *MainLogList,*MAIN_PANED,*MAIN_PANED2;
+extern GtkWidget *MainLogList,*MAIN_PANED,*MAIN_PANED2,*MAIN_PANED1;
 extern GtkAdjustment *ProgressBarValues;
 extern GtkWidget *ProgressOfDownload;
 extern GtkWidget *MainStatusBar;
 extern GtkWidget *MainWindow;
 extern GdkGC *MainWindowGC;
-extern tFaceLimits *FaceForLimits;
 extern GtkWidget *ContainerForCList;
 extern gint StatusBarContext;
-extern GtkCList *FSearchCList;
+extern GtkTreeView *FSearchView;
+extern GtkTreeView *FSearchView2;
+extern d4xQsTree *D4X_QVT;
+
+struct d4xDisplayLogInfo{
+	GtkTreeView *view;
+	tLog *log;
+	tDownload *papa;
+	GtkWidget *buttonsbar;
+	GtkWidget *buttons[10];
+	int curbutton;
+};
+
+extern  d4xDisplayLogInfo D4X_LOG_DISPLAY;
+
 
 void main_menu_speed_prepare();
 void main_menu_completed_empty();
@@ -74,7 +85,20 @@ void continue_downloads(...);
 void my_main_quit(...);
 char *old_clipboard_content();
 
-void d4x_mw_clipboard_set(char *str);
+void d4x_mw_clipboard_set(const char *str);
 char *d4x_mw_clipboard_get();
-void d4x_normalize_coords(gint *x,gint *y);
+void d4x_normalize_coords(gint *x,gint *y,gint width=0,gint heigh=0);
+void d4x_main_switch_log(tDownload *dwn);
+void d4x_main_log_del_string();
+void d4x_main_log_add_string(tLogString *str);
+void d4x_main_offline_mode();
+void d4x_vertical_toolbar_change_theme();
+
+void list_dnd_drop_internal(GtkWidget *widget,
+			    GdkDragContext *context,
+			    gint x, gint y,
+			    GtkSelectionData *selection_data,
+			    guint info,
+			    guint time);
+
 #endif
