@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -45,7 +45,7 @@ char *FTP_EXIST_DATA[]={
 int tFtpClient::accepting() {
 	if (passive) return RVALUE_OK;
 	DSFlag=1;
-	if (DataSocket->accepting(hostname)) {
+	if (DataSocket->accepting(hostname.c_str())) {
 		Status=STATUS_TRIVIAL;
 		LOG->log(LOG_ERROR,_("Accepting faild"));
 		return RVALUE_TIMEOUT;
@@ -53,7 +53,7 @@ int tFtpClient::accepting() {
 	return RVALUE_OK;
 };
 
-int  tFtpClient::send_command(char * comm,char *argv) {
+int  tFtpClient::send_command(const char * comm,const char *argv) {
 	DBC_RETVAL_IF_FAIL(comm!=NULL,RVALUE_OK);
 
 	char *data=NULL;
@@ -78,9 +78,9 @@ int  tFtpClient::send_command(char * comm,char *argv) {
 	return RVALUE_OK;
 };
 
-int tFtpClient::read_data(char *where,fsize_t len) {
+fsize_t tFtpClient::read_data(char *where,fsize_t len) {
 	DBC_RETVAL_IF_FAIL(where!=NULL,RVALUE_TIMEOUT);
-	int all=DataSocket->rec_string(where,len,timeout);
+	fsize_t all=DataSocket->rec_string(where,len,timeout);
 	if (socket_err_handler(all)) {
 		LOG->log(LOG_WARNING,_("Data connection lost!"));
 		vdisconnect();
@@ -97,9 +97,11 @@ int tFtpClient::read_control() {
 		return RVALUE_TIMEOUT;
 	};
 	tString *log=CTRL->last();
-	if (log)
-		LOG->log(LOG_FROM_SERVER,log->body);
-	else{
+	if (log){
+		gchar *text_utf=g_convert_with_fallback(log->body,-1,"UTF-8","ISO8859-1",NULL,NULL,NULL,NULL);
+		LOG->log(LOG_FROM_SERVER,text_utf);
+		g_free(text_utf);
+	}else{
 //		vdisconnect();
 		return RVALUE_TIMEOUT;
 	};
@@ -172,21 +174,28 @@ int tFtpClient::last_answer(char *first) {
 	return 0;
 };
 
-int tFtpClient::rest(int offset) {
+int tFtpClient::rest(fsize_t offset) {
 	ReGet=1;
-	if (offset && ReGet) {
+	if (offset || CUR_REST!=0) {
 		char data[MAX_LEN];
-		sprintf(data,"%i",offset);
+		sprintf(data,"%lli",offset);
 		send_command("REST",data);
 		int a=analize_ctrl(1,&FTP_REST_OK);
 		if (a==RVALUE_TIMEOUT) return(a);
 		if (a!=RVALUE_OK){
 			ReGet=0;
 			LOG->log(LOG_WARNING,_("Reget is not supported!!!"));
-		};
+			CUR_REST=0;
+		}else
+			CUR_REST=offset;
 	};
 	return RVALUE_OK;
 };
+
+int tFtpClient::force_reget(){
+	return(rest(100));
+};
+
 //**************************************************/
 
 tFtpClient::tFtpClient():tClient(){
@@ -198,9 +207,10 @@ tFtpClient::tFtpClient():tClient(){
 	METHOD_TO_LIST=0;
 	DataSocket=new tSocket;
 	log_flag=0;
+	CUR_REST=0;
 };
 
-tFtpClient::tFtpClient(tCfg *cfg,tSocket *ctrl=NULL):tClient(cfg,ctrl){
+tFtpClient::tFtpClient(tCfg *cfg,tSocket *ctrl):tClient(cfg,ctrl){
 	passive=cfg->passive;
 	TEMP_SIZE=OLD_SIZE=0;
 	CTRL=new tStringList;
@@ -214,9 +224,10 @@ tFtpClient::tFtpClient(tCfg *cfg,tSocket *ctrl=NULL):tClient(cfg,ctrl){
 					    cfg->socks_pass.get());
 	}else
 		DataSocket=new tSocket;
+	CUR_REST=0;
 };
 
-void tFtpClient::init(char *host,tWriterLoger *log,int prt,int time_out) {
+void tFtpClient::init(const std::string &host,tWriterLoger *log,int prt,int time_out) {
 	tClient::init(host,log,prt,time_out);
 	DSFlag=0;
 	BuffSize=BLOCK_READ;
@@ -227,6 +238,7 @@ void tFtpClient::init(char *host,tWriterLoger *log,int prt,int time_out) {
 
 int tFtpClient::reinit() {
 	ReGet=1;
+	CUR_REST=0;
 	int rvalue=0;
 	quit();
 	vdisconnect();
@@ -237,17 +249,17 @@ int tFtpClient::reinit() {
 };
 
 
-int tFtpClient::registr(char *user,char *password) {
+int tFtpClient::registr(const std::string &user,const std::string &password) {
 	username=user;
 	userword=password;
 	return 0;
 };
 
 int tFtpClient::connect() {
-	send_command("USER",username);
+	send_command("USER",username.c_str());
 	if (analize_ctrl(sizeof(FTP_LOGIN_OK)/sizeof(char *),FTP_LOGIN_OK)) return -1;
 	if (analize(FTP_PASS_OK)){
-		send_command("PASS",userword);
+		send_command("PASS",userword.c_str());
 		if (analize_ctrl(1,&FTP_PASS_OK)) return -2;
 	};
 	log_flag=1;
@@ -309,7 +321,7 @@ int tFtpClient::stand_data_connection() {
 			(addr>>24)&0xff,(addr>>16)&0xff,(addr>>8)&0xff,(addr)&0xff,
 			(port>>8)&0xff,(port)&0xff);
 		send_command("PORT",data);
-		if (analize_ctrl(1,&FTP_PORT_OK)) {
+		if (analize_ctrl(1,&FTP_PORT_OK) || Status==STATUS_CMD_ERR) {
 			passive=1;
 			Status=STATUS_TRIVIAL;
 			return -1;
@@ -324,7 +336,7 @@ void tFtpClient::vdisconnect(){
 	log_flag=0;
 };
 
-int tFtpClient::change_dir(char *where) {
+int tFtpClient::change_dir(const char *where) {
 	if (where !=NULL && strlen(where)) {
 		send_command("CWD",where);
 		return analize_ctrl(1,&FTP_CWD_OK);
@@ -337,7 +349,7 @@ void _ftp_filename_destroy_(void *a){
 	delete[] b;
 };
 
-fsize_t tFtpClient::get_size(char *filename,tStringList *list) {
+fsize_t tFtpClient::get_size(const char *filename,tStringList *list) {
 //	DBC_RETVAL_IF_FAIL(filename!=NULL,RVALUE_OK);
 	DBC_RETVAL_IF_FAIL(list!=NULL,RVALUE_OK);
 
@@ -382,18 +394,21 @@ fsize_t tFtpClient::get_size(char *filename,tStringList *list) {
 	return(rvalue);
 };
 
-int tFtpClient::get_file_from(char *what,unsigned int begin,fsize_t len) {
+fsize_t tFtpClient::get_file_from(const char *what,fsize_t begin,fsize_t len) {
 	DBC_RETVAL_IF_FAIL(what!=NULL,RVALUE_OK);
-	int rvalue=0;;
+#ifdef DEBUG_ALL
+	LOG->log_printf(LOG_OK,"tFtpClient::get_file_from(%s,%ll,%ll)",what,begin,len);
+#endif //DEBUG ALL
+	fsize_t rvalue=0;;
 	FileLoaded=begin;
-	char data[25];
-	data[0]=0;
-	sprintf(data,"%u",begin);
 	send_command("TYPE","I");
 	if ((rvalue=analize_ctrl(1,&FTP_PORT_OK))) return(rvalue);
 	if ((rvalue=rest(begin))) return rvalue;
 	if (!ReGet) {
-		if (!RETRY_IF_NO_REGET) return(-1);
+#ifdef DEBUG_ALL
+		LOG->log_printf(LOG_OK,"Warning! problems with resuming!",what,begin,len);
+#endif //DEBUG ALL
+		if (!RETRY_IF_NO_REGET) return(RVALUE_UNSPEC_ERR);
 		begin=0;
 		LOG->shift(0);
 		FileLoaded=0;
@@ -412,7 +427,7 @@ int tFtpClient::get_file_from(char *what,unsigned int begin,fsize_t len) {
 	if (OLD_SIZE && OLD_SIZE>TEMP_SIZE){
 		LOG->log(LOG_WARNING,_("Probably file was changed on server!"));
 		ReGet=0;
-		if (!RETRY_IF_NO_REGET) return(-1);
+		if (!RETRY_IF_NO_REGET) return(RVALUE_UNSPEC_ERR);
 		send_command("REST","0");
 		LOG->shift(0);
 		FileLoaded=0;
@@ -424,15 +439,17 @@ int tFtpClient::get_file_from(char *what,unsigned int begin,fsize_t len) {
 	if (Status) return RVALUE_TIMEOUT;
 	DSize=0;
 	int complete;
-	int llen=len;
+	fsize_t llen=len;
+	LOG->log_printf(LOG_WARNING,"low level loading started from %ll load %ll bytes",begin,len);
 	do {
-		if ((complete=tClient::read_data())<0) {
+		if ((complete=tClient::read_data(BLOCK_READ))<0) {
 			LOG->log(LOG_WARNING,_("Data connection closed."));
 			break;
 		};
 		if (len && FillSize>llen) FillSize=llen;
 		FileLoaded+=FillSize;
 		if (write_buffer()) {
+			LOG->log(LOG_ERROR,_("Error have happened during writing buffer to disk!"));
 			Status=STATUS_FATAL;
 			break;
 		};
@@ -440,22 +457,43 @@ int tFtpClient::get_file_from(char *what,unsigned int begin,fsize_t len) {
 			llen -=FillSize;
 			if (llen==0){
 				LOG->log(LOG_OK,_("Requested size was loaded"));
-				DataSocket->flush(); /*read data in socket
+				DataSocket->flush(); /*read data from socket
 						      to avoid "brocken pipe"
 						      on linux;*/
 				DataSocket->down();
 				analize_ctrl(1,&FTP_READ_OK);
 				Status=0;
+				LOG->log_printf(LOG_WARNING,"loading end. loaded %ll bytes",DSize);
 				return DSize;
 			};
 		};
+		if (LOG->is_overlaped()){
+			LOG->log(LOG_OK,_("Segment was loaded! Complete this thread."));
+			DataSocket->down();
+//			send_command("ABOR",NULL);
+			Status=0;
+			LOG->log_printf(LOG_WARNING,"loading end. loaded %ll bytes",DSize);
+			return DSize;
+		};
 	} while (complete!=0);
+	if (complete==0) LOG->log(LOG_WARNING,_("EOF recieved from server!"));
+#ifdef DEBUG_ALL
+	LOG->log_printf(LOG_OK,"Out of the cicle with DSize=%ll",DSize);
+#endif //DEBUG ALL
 	DataSocket->down(); // to prevent next ideas from guys of wu-ftpd's team
-	if (Status) return DSize;
-	if (analize_ctrl(1,&FTP_READ_OK)){
-		if (len && llen==0) return DSize;
-		return(RVALUE_UNSPEC_ERR);
+	LOG->log_printf(LOG_WARNING,"loading end. loaded %ll bytes",DSize);
+	if (Status){
+#ifdef DEBUG_ALL
+		LOG->log_printf(LOG_OK,"exiting from tFtpDownload with %ll",DSize);
+#endif //DEBUG ALL
+		return DSize;
 	};
+	analize_ctrl(1,&FTP_READ_OK);
+//	if (analize_ctrl(1,&FTP_READ_OK) &&  (len==0 || llen))
+//		return(RVALUE_UNSPEC_ERR);
+#ifdef DEBUG_ALL
+	LOG->log_printf(LOG_OK,"exiting from tFtpDownload with %ll",DSize);
+#endif //DEBUG ALL
 	return DSize;
 };
 
