@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified 
  *	program but you can distribute unmodified program.
@@ -22,6 +22,7 @@
 #include <string.h>
 #include <strings.h>
 #include <time.h>
+#include "signal.h"
 
 //****************************************/
 void ftp_extract_link(char *src,char *dst) {
@@ -107,6 +108,9 @@ time_t ftp_date_from_str(char *src) {
 	tmp=extract_string(src,data,5);
 	if (is_string(data)){
 		tmp=extract_string(tmp,data);
+		if (!is_string(data)){
+			tmp=extract_string(src,data,4);
+		};
 	};
 
 	time_t NOW=time(NULL);
@@ -176,11 +180,11 @@ void ftp_cut_string_list(char *src,tFileInfo *dst,int flag) {
 		char *tmp;
 		if (!is_string(str1)){
 			tmp=skip_strings(rsrc,8);
-			sscanf(rsrc,"%s %u %s %s %li %s %u %s %s",
+			sscanf(rsrc,"%s %u %s %s %lli %s %u %s %s",
 	       		str1,&par1,str1,str1,&dst->size,str1,&par1,str1,name);
 		}else{
 			tmp=skip_strings(rsrc,7);
-			sscanf(rsrc,"%s %u %s %li %s %u %s %s",
+			sscanf(rsrc,"%s %u %s %lli %s %u %s %s",
 	       		str1,&par1,str1,&dst->size,str1,&par1,str1,name);
 		};
 		dst->type=ftp_type_from_str(rsrc);
@@ -212,7 +216,7 @@ void ftp_cut_string_list(char *src,tFileInfo *dst,int flag) {
 				dst->type=T_DIR;
 				dst->perm=0775;
 			}else{
-				sscanf(str1,"%li",&(dst->size));
+				sscanf(str1,"%lli",&(dst->size));
 				dst->type=T_FILE;
 				dst->perm=0664;
 			};
@@ -260,7 +264,7 @@ int tFtpDownload::change_dir() {
 		return(rvalue);
 	};
 	*/
-	if ((rvalue=FTP->change_dir(ADDR.path.get()))){
+	if ((rvalue=FTP->change_dir(ADDR.path.c_str()))){
 		print_error(ERROR_CWD);
 		return(rvalue);
 	};
@@ -275,7 +279,6 @@ tFtpDownload::tFtpDownload():tDownloader(){
 	DIR=list=NULL;
 	RetrNum=0;
 	DONT_CWD=0;
-	TMP_FILEPATH=NULL;
 };
 
 tFtpDownload::tFtpDownload(tWriterLoger *log):tDownloader(log){
@@ -283,7 +286,6 @@ tFtpDownload::tFtpDownload(tWriterLoger *log):tDownloader(log){
 	DIR=list=NULL;
 	RetrNum=0;
 	DONT_CWD=0;
-	TMP_FILEPATH=NULL;
 };
 
 void tFtpDownload::dont_cwd(){
@@ -305,18 +307,20 @@ int tFtpDownload::reconnect() {
 		};
 		RetrNum++;
 		print_error(ERROR_ATTEMPT);
+		tDownloader::reconnect();
 		if (RetrNum>1) {
 			LOG->log(LOG_OK,_("Sleeping"));
+			d4x::USR1Off2On sig_unlocker_;
 			sleep(config.time_for_sleep+1);
 		};
 		if (FTP->reinit()==0){
 			success=0;
-			if (config.proxy_user.get() && config.proxy_pass.get()){
-				FTP->registr(config.proxy_user.get(), config.proxy_pass.get());
+			if (config.proxy.ftp_user.get() && config.proxy.ftp_pass.get()){
+				FTP->registr(config.proxy.ftp_user.get(), config.proxy.ftp_pass.get());
 				success=FTP->connect();
 			};
 			if (success==0){
-				FTP->registr(ADDR.username.get(),ADDR.pass.get());
+				FTP->registr(ADDR.user,ADDR.pass);
 				success=FTP->connect();
 			};
 		};
@@ -325,24 +329,23 @@ int tFtpDownload::reconnect() {
 	return 0;
 };
 
-void tFtpDownload::init_download(char *path,char *file) {
-	ADDR.file.set(file);
-	if (path && *path!='~' && *path!='/'){
-		path=sum_strings("/",path,NULL);
-		ADDR.path.set(path);
-		delete[] path;
+void tFtpDownload::init_download(const std::string &path,const std::string &file) {
+	ADDR.file=file;
+	if (path[0]!='~' && path[0]!='/'){
+		ADDR.path="/";
+		ADDR.path/=path;
 	}else
-		ADDR.path.set(path);
+		ADDR.path=path;
 };
 
-int tFtpDownload::init(tAddr *hostinfo,tCfg *cfg,tSocket *s=NULL) {
+int tFtpDownload::init(const d4x::URL &hostinfo,tCfg *cfg,tSocket *s) {
 	FTP=new tFtpClient(cfg);
 	RetrNum=0;
-	ADDR.copy(hostinfo);
-	if (ADDR.username.get()==NULL)
-		ADDR.username.set(DEFAULT_USER);
-	if (ADDR.pass.get()==NULL)
-		ADDR.pass.set(CFG.ANONYMOUS_PASS);
+	ADDR=hostinfo;
+	if (ADDR.user.empty())
+		ADDR.user=DEFAULT_USER;
+	if (ADDR.pass.empty())
+		ADDR.pass=CFG.ANONYMOUS_PASS;
 	DIR=NULL;
 	list=NULL;
 
@@ -353,18 +356,16 @@ int tFtpDownload::init(tAddr *hostinfo,tCfg *cfg,tSocket *s=NULL) {
 	};
 	config.copy_proxy(cfg);
 
-	if (config.proxy_host.get() && config.proxy_port) {
-		FTP->init(config.proxy_host.get(),LOG,config.proxy_port,config.timeout);
+	if (config.proxy.type==0 && config.proxy.ftp_host.get()
+	    && config.proxy.ftp_host.get()[0] && config.proxy.ftp_port) {
+		FTP->init(config.proxy.ftp_host.get(),LOG,config.proxy.ftp_port,config.timeout);
 		char port[MAX_LEN];
 		port[0]=0;
 		if (ADDR.port!=get_port_by_proto(D_PROTO_FTP))
 			sprintf(port,":%i",ADDR.port);
-		char *temp=sum_strings(ADDR.username.get(),"@",
-				       ADDR.host.get(),port,NULL);
-		ADDR.username.set(temp);
-		delete[] temp;
+		ADDR.user=ADDR.user+"@"+ADDR.host+port;
 	} else
-		FTP->init(ADDR.host.get(),LOG,ADDR.port,config.timeout);
+		FTP->init(ADDR.host,LOG,ADDR.port,config.timeout);
 	FTP->set_passive(config.passive);
 	FTP->set_retry(config.retry);
 	FTP->set_dont_set_quit(config.dont_send_quit);
@@ -372,30 +373,42 @@ int tFtpDownload::init(tAddr *hostinfo,tCfg *cfg,tSocket *s=NULL) {
 		FTP->import_ctrl_socket(s);
 		CWDFlag=0;
 		RetrNum=1;
+		tDownloader::reconnect();
 		return(0);
 	};
-	return reconnect();
+	while(reconnect()==0){
+		if (cfg->split==0 || FTP->force_reget()==0)
+			return(0);
+	};
+	return(-1);
 };
 
 tStringList *tFtpDownload::dir() {
 	return DIR;
 };
 
-int tFtpDownload::ftp_get_size(tStringList *l){
-	if (ADDR.mask)
+int tFtpDownload::ftp_get_size_no_sdc(tStringList *l){
+	if (ADDR.mask){
 		return(FTP->get_size(NULL,l));
-	if (DONT_CWD){
-		if (TMP_FILEPATH) delete[] TMP_FILEPATH;
-		TMP_FILEPATH=sum_strings("/",ADDR.path.get(),"/",
-					 ADDR.file.get(),NULL);
-		normalize_path(TMP_FILEPATH);
-		return(FTP->get_size(TMP_FILEPATH,l));
 	};
-	return(FTP->get_size(ADDR.file.get(),l));
+	if (DONT_CWD){
+		TMP_FILEPATH=std::string("/")+ADDR.path;
+		TMP_FILEPATH/=ADDR.file;
+		return(FTP->get_size(TMP_FILEPATH.c_str(),l));
+	};
+	return(FTP->get_size(ADDR.file.c_str(),l));
+};
+
+int tFtpDownload::ftp_get_size(tStringList *l){
+	if (FTP->stand_data_connection()){
+		print_error(ERROR_DATA_CONNECT);
+		return(-1);
+	};
+	return(ftp_get_size_no_sdc(l));
 };
 
 int tFtpDownload::is_dir(){
-	if (FTP->change_dir(ADDR.file.get())==0)
+	if (FTP->change_dir(ADDR.file.c_str())==0)
 		return(1);
 	return(0);
 };
@@ -408,10 +421,8 @@ fsize_t tFtpDownload::ls_answer_short(){
 		/* try to CWD */
 		if (is_dir()){
 			D_FILE.type=T_DIR;
-			char *a=compose_path(ADDR.path.get(),ADDR.file.get());
-			ADDR.path.set(a);
-			ADDR.file.set("");
-			delete[] a;
+			ADDR.path/=ADDR.file;
+			ADDR.file.clear();
 		}else
 			D_FILE.type=T_FILE;
 		return 0;
@@ -427,19 +438,19 @@ fsize_t tFtpDownload::ls_answer_short(){
 		D_FILE.perm=S_IRUSR|S_IWUSR|S_IXUSR;
 		return 0;
 	};
-	ftp_cut_string_list(last->body,&D_FILE,0);
-	LOG->log_printf(LOG_OK,_("Length is %i"),D_FILE.size);
+	ftp_cut_string_list(last->body,&D_FILE,1);
+	LOG->log_printf(LOG_OK,_("Length is %ll"),D_FILE.size);
 	return D_FILE.size;
 };
 
 fsize_t tFtpDownload::ls_answer_long(){
 	tString *last=list->last();
 	while(last){
-		if (strstr(last->body,ADDR.file.get()))
+		if (strstr(last->body,ADDR.file.c_str()))
 			break;
 		last=(tString*)(last->next);
 	};
-	if (last==NULL || is_dir()){
+	if (is_dir() || ADDR.mask){
 		LOG->log(LOG_WARNING,_("This is a directory!"));
 		D_FILE.size=1;
 		if (DIR) delete DIR;
@@ -449,12 +460,20 @@ fsize_t tFtpDownload::ls_answer_long(){
 		D_FILE.perm=S_IRUSR|S_IWUSR;
 		return 1;
 	};
+	if (last==NULL){
+		LOG->log(LOG_WARNING,_("No such file or directory!"));
+		return(-1);
+	};
 	ftp_cut_string_list(last->body,&D_FILE,0);
-	LOG->log_printf(LOG_OK,_("Length is %i"),D_FILE.size);
+	LOG->log_printf(LOG_OK,_("Length is %ll"),D_FILE.size);
 	D_FILE.type=T_FILE;
 	return(D_FILE.size);
 };
 
+fsize_t tFtpDownload::get_size_only() {
+	return(get_size());
+};
+
 fsize_t tFtpDownload::get_size() {
 	if (!list) {
 		list=new tStringList;
@@ -462,17 +481,22 @@ fsize_t tFtpDownload::get_size() {
 	} else list->done();
 	while (1) {
 		if (!change_dir()) {
-			if (!FTP->stand_data_connection()) {
-				int a=ftp_get_size(list);
+			int a=ftp_get_size(list);
+			if (a==0 && list->count()<=2){
+				fsize_t sz=ls_answer_short();
+				if (ADDR.file==D_FILE.name.get() ||
+				    FTP->METHOD_TO_LIST==1)
+					return(sz);
+				FTP->METHOD_TO_LIST=1;
+				list->done();
+				a=ftp_get_size(list);
 				if (a==0 && list->count()<=2)
 					return(ls_answer_short());
-				if (a==0 && list->count()>2) {
-					return(ls_answer_long());
-				};
-				LOG->log(LOG_WARNING,_("Couldn't get size :(("));
-			} else {
-				print_error(ERROR_DATA_CONNECT);
 			};
+			if (a==0 && list->count()>2) {
+				return(ls_answer_long());
+			};
+			LOG->log(LOG_WARNING,_("Couldn't get size :(("));
 		};
 		if (FTP->get_status()==STATUS_CMD_ERR ||
 		    FTP->get_status()==STATUS_UNSPEC_ERR){
@@ -499,7 +523,7 @@ int tFtpDownload::download_dir() {
 						DIR->done();
 					};
 					Status=D_DOWNLOAD;
-					ind=ftp_get_size(DIR);
+					ind=ftp_get_size_no_sdc(DIR);
 					if (ind==0) {
 						LOG->log(LOG_OK,_("Listing was loaded"));
 						return 0;
@@ -525,6 +549,9 @@ int tFtpDownload::download_dir() {
 
 int tFtpDownload::download(fsize_t len) {
 	int rvalue=0;
+#ifdef DEBUG_ALL
+	LOG->log_printf(LOG_OK,"tFtpDownload::download(%ll)",len);
+#endif //DEBUG ALL
 	switch (D_FILE.type){
 	case T_DIR: {
 		rvalue=download_dir();
@@ -552,20 +579,23 @@ int tFtpDownload::download(fsize_t len) {
 					Status=D_DOWNLOAD;
 					fsize_t to_load=len>0?length_to_load-LOADED:0;
 					if (DONT_CWD){
-						if (TMP_FILEPATH) delete[] TMP_FILEPATH;
-						TMP_FILEPATH=sum_strings("/",ADDR.path.get(),"/",
-									 ADDR.file.get(),NULL);
-						normalize_path(TMP_FILEPATH);
-						ind=FTP->get_file_from(TMP_FILEPATH,LOADED,to_load);;
+						TMP_FILEPATH=std::string("/")+ADDR.path;
+						TMP_FILEPATH/=ADDR.file;
+						ind=FTP->get_file_from(TMP_FILEPATH.c_str(),LOADED,to_load);
 					}else
-						ind=FTP->get_file_from(ADDR.file.get(),LOADED,to_load);
+						ind=FTP->get_file_from(ADDR.file.c_str(),LOADED,to_load);
+#ifdef DEBUG_ALL
+					LOG->log_printf(LOG_OK,"return to tFtpDownload::download with ind=%ll",ind);
+#endif //DEBUG ALL
 					if (!FTP->test_reget()){
 						if (config.retry==0) break;
 						StartSize=LOADED=0;
 					};
 					if (ind>0) {
 						LOADED+=ind;
-						LOG->log_printf(LOG_OK,_("%i bytes loaded."),ind);
+//						LOG->log_printf(LOG_OK,_("%ll bytes loaded."),ind);
+						LOG->log_printf(LOG_OK,"%ll bytes loaded.(LOADED=%ll,len=%ll,FTP->get_status()=%i)",
+								ind,LOADED,len,FTP->get_status());
 					};
 					if (!FTP->get_status()) {
 						rvalue=0;
@@ -589,6 +619,9 @@ int tFtpDownload::download(fsize_t len) {
 		};
 	};
 	};
+#ifdef DEBUG_ALL
+	LOG->log_printf(LOG_OK,"exit from tFtpDownload::download with %i",rvalue);
+#endif //DEBUG ALL
 	return rvalue;
 };
 
@@ -613,8 +646,13 @@ fsize_t tFtpDownload::get_start_size() {
 	return(StartSize);
 };
 
+char *tFtpDownload::get_new_url() {
+	return(copy_string(D_FILE.body.get()));
+};
+
 int tFtpDownload::reget() {
-	return FTP->test_reget();
+	if (FTP) return FTP->test_reget();
+	return(1);
 };
 
 void tFtpDownload::done() {
@@ -630,5 +668,4 @@ tFtpDownload::~tFtpDownload() {
 	if (FTP) delete(FTP);
 	if (DIR) delete(DIR);
 	if (list) delete(list);
-	if (TMP_FILEPATH) delete[] TMP_FILEPATH;
 };
