@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -20,6 +20,8 @@
 #include <string.h>
 #include <strings.h>
 #include <ctype.h>
+#include "signal.h"
+#include "sslsocket.h"
 
 enum HTTP_ANSWERS_ENUM{
 	H_CONTENT_LENGTH,
@@ -29,7 +31,11 @@ enum HTTP_ANSWERS_ENUM{
 	H_ACCEPT_RANGE,
 	H_ETAG,
 	H_WWW_AUTHENTICATE,
-	H_LAST_MODIFIED
+	H_LAST_MODIFIED,
+	H_SET_COOKIE,
+	H_TRANSFER_ENCODING,
+	H_CONNECTION,
+	H_PROXY_CONNECTION
 };
 
 char *http_answers[]={
@@ -40,7 +46,11 @@ char *http_answers[]={
 	"accept-ranges:",
 	"etag:",
 	"www-authenticate:",
-	"last-modified:"
+	"last-modified:",
+	"set-cookie:",
+	"transfer-encoding:",
+	"connection:",
+	"proxy-connection:"
 };
 
 
@@ -82,7 +92,7 @@ d4xContentDisposition::d4xContentDisposition(char *httpstr){
 				r_date=ctime_to_time(value);
 				break;
 			case CDE_SIZE:
-				sscanf(value,"%li",&size);
+				sscanf(value,"%lli",&size);
 				break;
 			};
 			delete[] value;
@@ -115,6 +125,7 @@ char *d4xContentDisposition::get_value(char *s){
 	if (sqs==NULL) sqs=tmp+strlen(tmp)-1;
 	else sqs-=1;
 	int len=sqs-tmp;
+	if (len<=0) return(NULL);
 	char *rval=copy_string2(tmp,len);
 	for (int i=len-1;i>=0;i++){
 		if (isspace(rval[i])) rval[i]=0;
@@ -149,7 +160,6 @@ tHttpDownload::tHttpDownload():tDownloader(){
 	HTTP=NULL;
 	OldETag=ETag=Auth=NULL;
 	content_type=NULL;
-	REQUESTED_URL=NULL;
 	content_disp=NULL;
 };
 
@@ -158,7 +168,6 @@ tHttpDownload::tHttpDownload(tWriterLoger *log):tDownloader(log){
 	HTTP=NULL;
 	OldETag=ETag=Auth=NULL;
 	content_type=NULL;
-	REQUESTED_URL=NULL;
 	content_disp=NULL;
 };
 
@@ -172,10 +181,15 @@ void tHttpDownload::print_error(int error_code){
 	};
 };
 
-int tHttpDownload::init(tAddr *hostinfo,tCfg *cfg,tSocket *s=NULL) {
+int tHttpDownload::init(const d4x::URL &hostinfo,tCfg *cfg,tSocket *s) {
+	Persistent=0;
+#ifdef HAVE_SSL
+	HTTP=new tHttpClient(cfg,(hostinfo.proto==D_PROTO_HTTPS && s==0) ? new d4x::SSLSocket:0);
+#else
 	HTTP=new tHttpClient(cfg);
+#endif //HAVE_SSL	
 	RetrNum=0;
-	ADDR.copy(hostinfo);
+	ADDR=hostinfo;
 	answer=NULL;
 	ETag=NULL;
 	Auth=NULL;
@@ -185,12 +199,18 @@ int tHttpDownload::init(tAddr *hostinfo,tCfg *cfg,tSocket *s=NULL) {
 		config.retry=0;
 		config.rollback=0;
 	};
-	HTTP->init(ADDR.host.get(),LOG,ADDR.port,config.timeout);
+	HTTP->init(ADDR.host,LOG,ADDR.port,config.timeout);
 	config.user_agent.set(cfg->user_agent.get());
 	config.referer.set(cfg->referer.get());
 	HTTP->set_user_agent(config.user_agent.get(),config.referer.get());
-	HTTP->registr(ADDR.username.get(),ADDR.pass.get());
+	HTTP->registr(ADDR.user,ADDR.pass);
 	REQUESTED_URL=make_name();
+	if (s){
+		HTTP->import_ctrl_socket(s);
+		RetrNum=1;
+		tDownloader::reconnect();
+		return(0);
+	};
 	return reconnect();
 };
 
@@ -205,9 +225,11 @@ int tHttpDownload::reconnect() {
 		};
 		RetrNum++;
 		print_error(ERROR_ATTEMPT);
+		tDownloader::reconnect();
 		HTTP->down();
 		if (RetrNum>1) {
 			LOG->log(LOG_WARNING,_("Sleeping"));
+			d4x::USR1Off2On sig_unlocker;
 			sleep(config.time_for_sleep+1);
 		};
 		if (HTTP->reinit()==0)
@@ -235,6 +257,14 @@ tStringList *tHttpDownload::dir() {
 	return answer;
 };
 
+int tHttpDownload::persistent(){
+	return(Persistent);
+};
+
+fsize_t tHttpDownload::another_way_get_size() {
+	return 0;
+};
+
 fsize_t tHttpDownload::analize_answer() {
 	/*  Here we need few analisation of http answer
 	 *	AcceptRanges: bytes
@@ -242,6 +272,8 @@ fsize_t tHttpDownload::analize_answer() {
 	 *	Content-Range: bytes (int)-(int)/(int)
 	 *	Last-Modified: 
 	 */
+	if (HTTP->HTTP_SUBVER==1 || HTTP->HTTP_VER>1)
+		Persistent=1;
 	ETagChanged=0;
 	OldETag=ETag;
 	ETag=NULL;
@@ -250,6 +282,7 @@ fsize_t tHttpDownload::analize_answer() {
 	fsize_t rvalue=0;
 	ReGet=0;
 	MustNoReget=0;
+	HTTP->CHUNKED=0;
 	while(temp) {
 		char *STR=NULL;
 		unsigned int i;
@@ -273,7 +306,7 @@ fsize_t tHttpDownload::analize_answer() {
 			break;
 		};
 		case H_CONTENT_LENGTH:{
-			sscanf(temp->body+strlen(STR),"%li",&rvalue);
+			sscanf(temp->body+strlen(STR),"%Li",&rvalue);
 			break;
 		};
 		case H_CONTENT_RANGE:{
@@ -284,8 +317,8 @@ fsize_t tHttpDownload::analize_answer() {
 					a+=1;
 			}else
 				a=temp->body+strlen(STR);
-			int b[3];
-			if (sscanf(a,"%i-%i/%i",&b[0],&b[1],&b[2])==3){
+			fsize_t b[3];
+			if (sscanf(a,"%Li-%Li/%Li",&b[0],&b[1],&b[2])==3){
 				ReGet=1;
 				rvalue=b[2]-b[0];
 			};
@@ -303,7 +336,7 @@ fsize_t tHttpDownload::analize_answer() {
 			ETag=extract_from_prefixed_string(temp->body,STR);
 			char *tmp=index(ETag,':');
 			if (tmp) *tmp=0;
-			if (OldETag) {
+			if (OldETag && config.ihate_etag==0) {
 				if (!equal(OldETag,ETag)) {
 					MustNoReget=ETagChanged=1;
 				};
@@ -321,11 +354,33 @@ fsize_t tHttpDownload::analize_answer() {
 			};
 			break;
 		};
+		case H_SET_COOKIE:{
+// Set-Cookie: zzzplayuniq=free3-chi-48-2001-Dec-28_04:57:07; expires=Sat, 31 Dec 2005 00:00:00 GMT; path=/; domain=.playboy.com;
+			/* FIXME: to avoid lost cookies, parse answer before redirection */
+			tCookie *cookie=new tCookie;
+			if (cookie->parse(temp->body+strlen(STR),ADDR.host.c_str(),ADDR.path.c_str())==0){
+				LOG->cookie_set(cookie);
+			};
+			break;
+		};
+		case H_TRANSFER_ENCODING:{
+			if (strstr(temp->body,"chunked")){
+				HTTP->CHUNKED=1;
+			};
+			break;
+		};
+		case H_PROXY_CONNECTION:
+		case H_CONNECTION:{
+			if (strstr(temp->body,"close")){
+				Persistent=0;
+			};
+			break;
+		};
 		};
 		temp=answer->next();
 	};
 	if (rvalue>0)
-		LOG->log_printf(LOG_OK,_("Size for download is %i bytes"),rvalue);
+		LOG->log_printf(LOG_OK,_("Size for download is %ll bytes"),rvalue);
 	if (LOADED==0) ReGet=1;
 	if (MustNoReget) ReGet=0;
 	if (OldETag){ /* if etag disappeared */
@@ -335,44 +390,24 @@ fsize_t tHttpDownload::analize_answer() {
 	return rvalue;
 };
 
-char *tHttpDownload::make_name(){
-	char *parsed_name=unparse_percents(ADDR.file.get());
-	char *parsed_path=unparse_percents(ADDR.path.get());
-	char *rvalue=new char[strlen(parsed_path)+strlen(parsed_name)+
-			     (ADDR.params.get() ? strlen(ADDR.params.get())+1:0)+
-			     strlen("//")+1];
-	*rvalue=0;
-	strcat(rvalue,"/");
-	strcat(rvalue,parsed_path);
-	int len=strlen(parsed_path);
-	if (*parsed_path!=0 && len && parsed_path[len-1]!='/')
-		strcat(rvalue,"/");
-	strcat(rvalue,parsed_name);
-	if (ADDR.params.get()){
-		strcat(rvalue,"?");
-		strcat(rvalue,ADDR.params.get());
-	};
-	delete[] parsed_path;
-	delete[] parsed_name;
-	return rvalue;
+std::string tHttpDownload::make_name(){
+	return hexed_string((ADDR.path/ADDR.file)+(ADDR.params.empty()?std::string():std::string("?")+ADDR.params));
 };
 
-fsize_t tHttpDownload::get_size() {
+fsize_t tHttpDownload::get_size_only() {
 	if (!answer) {
 		answer=new tStringList;
 		answer->init(0);
 	};
 	while (1) {
 		answer->done();
-		HTTP->set_offset(LOADED);
+		HTTP->set_offset(0);
 		LOG->log(LOG_OK,_("Sending HTTP request..."));
-		int temp=HTTP->get_size(REQUESTED_URL,answer);
+		int temp=HTTP->get_size_only(REQUESTED_URL.c_str(),answer);
 		switch (temp){
 		case 0:{ // all ok
 			LOG->log(LOG_OK,_("Answer read ok"));
 			D_FILE.size=analize_answer();
-			if (ReGet && D_FILE.size>=0)
-				D_FILE.size+=LOADED;
 			return D_FILE.size;
 		};
 		case -2: // bad error code
@@ -388,12 +423,48 @@ fsize_t tHttpDownload::get_size() {
 	return -2;
 };
 
+fsize_t tHttpDownload::get_size() {
+	if (!answer) {
+		answer=new tStringList;
+		answer->init(0);
+	};
+	while (1) {
+		answer->done();
+		HTTP->set_offset(LOADED);
+		LOG->log(LOG_OK,_("Sending HTTP request..."));
+		int temp=HTTP->get_size(REQUESTED_URL.c_str(),answer);
+		switch (temp){
+		case 0:{ // all ok
+			LOG->log(LOG_OK,_("Answer read ok"));
+			D_FILE.size=analize_answer();
+			if (ReGet && D_FILE.size>=0)
+				D_FILE.size+=LOADED;
+			return D_FILE.size;
+		};
+		case -2: // bad error code
+			print_error(ERROR_BAD_ANSWER);
+			return -2;
+		case -1: // timeout
+			break;
+		case 1: // redirection
+			analize_answer(); // to avoid lost cookies;
+			return -1;
+		};
+		if (reconnect()) break;
+	};
+	return -2;
+};
+
+void tHttpDownload::pass_first_segment(){
+	if (HTTP) HTTP->pass_first_segment();
+};
+
 int tHttpDownload::download(fsize_t len) {
 	int success=1;
 	int first=1;
+	StartSize=LOADED;
 	fsize_t length_to_load=len>0?LOADED+len:0;
 	while(success) {
-		StartSize=LOADED;
 		if (!first) StartSize=rollback();
 		HTTP->set_offset(LOADED);
 		if (ReGet && len==0 && LOADED &&
@@ -426,12 +497,13 @@ int tHttpDownload::download(fsize_t len) {
 			fsize_t ind=HTTP->get_file_from(NULL,LOADED,to_load);
 			if (ind>=0) {
 				LOADED+=ind;
-				LOG->log_printf(LOG_OK,_("%i bytes loaded."),ind);
+				LOG->log_printf(LOG_OK,_("%ll bytes loaded."),ind);
 			};
 			break;
 		};
 		first=0;
 //		if (HTTP->get_status()==STATUS_FATAL) return -1;
+		if (LOG->is_overlaped()) break;
 		if (len>0 && LOADED>=length_to_load) break;
 		if (LOADED==D_FILE.size && D_FILE.size!=0) break;
 		if (HTTP->get_status()==0 && D_FILE.size==0) break;
@@ -465,60 +537,39 @@ void tHttpDownload::make_full_pathes(const char *path,char **name,char **guess)
 	DBC_RETURN_IF_FAIL(path!=NULL);
 	DBC_RETURN_IF_FAIL(name!=NULL);
 	DBC_RETURN_IF_FAIL(guess!=NULL);
-	char *full_path;
-	int flag=strlen(ADDR.file.get());
+	d4x::Path full_path(path);
 	if (config.http_recursing){
 		if (config.leave_server){
-			char *tmp=compose_path(path,ADDR.host.get());
-			full_path=compose_path(tmp,ADDR.path.get());
-			delete[] tmp;
+			full_path/=d4x::Path(ADDR.host)/ADDR.path;
 		}else
-			full_path=compose_path(path,ADDR.path.get());
-	}else
-		full_path=copy_string(path);
-	char *temp;
-	if (flag){
-		temp=(config.http_recursing && ADDR.params.get())?
-			sum_strings(".",ADDR.file.get(),"?",ADDR.params.get(),NULL):
-			sum_strings(".",ADDR.file.get(),NULL);
-		*name=compose_path(full_path,temp);
-		delete[] temp;
-		temp=(config.http_recursing && ADDR.params.get())?
-			sum_strings(ADDR.file.get(),"?",ADDR.params.get(),NULL):
-			sum_strings(ADDR.file.get(),NULL);
-		*guess=compose_path(full_path,temp);
-	}else{
-		temp=(config.http_recursing && ADDR.params.get())?
-			sum_strings(".",CFG.DEFAULT_NAME,"?",ADDR.params.get(),NULL):
-			sum_strings(".",CFG.DEFAULT_NAME,NULL);
-		*name=compose_path(full_path,temp);
-		delete[] temp;
-		temp=(config.http_recursing && ADDR.params.get())?
-			sum_strings(CFG.DEFAULT_NAME,"?",ADDR.params.get(),NULL):
-			sum_strings(CFG.DEFAULT_NAME,NULL);
-		*guess=compose_path(full_path,temp);
+			full_path/=ADDR.path;
 	};
-	delete[] full_path;
-	delete[] temp;
+	std::string temp=(ADDR.file.empty()?std::string(CFG.DEFAULT_NAME):ADDR.file);
+	if (config.http_recursing && !ADDR.params.empty())
+		temp+=std::string(config.quest_sign_replace?"_":"?")+ADDR.params;
+	*guess=copy_string((full_path/temp).c_str());
+	full_path/=std::string(".")+temp;
+	*name=copy_string(full_path.c_str());
 };
 
-void tHttpDownload::make_full_pathes(const char *path,char *another_name,char **name,char **guess) {
+void tHttpDownload::make_full_pathes(const char *path,const char *another_name,char **name,char **guess) {
 	DBC_RETURN_IF_FAIL(path!=NULL);
 	DBC_RETURN_IF_FAIL(another_name!=NULL);
 	DBC_RETURN_IF_FAIL(name!=NULL);
 	DBC_RETURN_IF_FAIL(guess!=NULL);
-	char *temp=sum_strings(".",another_name,NULL);
-	char *full_path=NULL;
-	if (config.http_recursing)
-		full_path=compose_path(path,ADDR.path.get());
-	else
-		full_path=copy_string(path);
-//	char *question_sign=index(full_path,'?');
-//	if (question_sign) *question_sign=0;
-	*name=compose_path(full_path,temp);
-	*guess=compose_path(full_path,another_name);
-	delete[] full_path;
-	delete[] temp;
+	d4x::Path full_path(path);
+	if (config.http_recursing){
+		if (config.leave_server){
+			full_path/=d4x::Path(ADDR.host)/ADDR.path;
+		}else
+			full_path/=ADDR.path;
+	};
+	std::string temp=std::string(".")+another_name;
+	if (config.http_recursing && !ADDR.params.empty())
+		temp+=std::string(config.quest_sign_replace?"_":"?")+ADDR.params;
+	full_path/=temp;
+	*name=copy_string(full_path.c_str());
+	*guess=copy_string(full_path.c_str());
 };
 
 void tHttpDownload::done() {
@@ -531,7 +582,6 @@ tSocket *tHttpDownload::export_ctrl_socket(){
 };
 
 tHttpDownload::~tHttpDownload() {
-	if (REQUESTED_URL) delete[] REQUESTED_URL;
 	if (HTTP) delete HTTP;
 	if (ETag) delete[] ETag;
 	if (OldETag) delete[] OldETag;
