@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -18,6 +18,7 @@
 #include <glib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <pwd.h>
 #include "var.h"
 #include "dbc.h"
 
@@ -150,6 +151,17 @@ char *sum_strings(const char *a,...){
 	return r;
 };
 
+char *compose_strings_array(int *len,const char *a,int la,const char *b){
+	DBC_RETVAL_IF_FAIL(a!=NULL,NULL);
+	int lb=(b?strlen(b)+1:0);
+	char *r=new char[la+lb];
+	memcpy(r,a,la);
+	if (b)
+		memcpy(r+la,b,lb);
+	*len=la+lb;
+	return r;
+};
+
 
 // next function reallocate memory for string and return new size
 int reallocate_string(char **what, int len){
@@ -193,13 +205,42 @@ void convert_int_to_2(int what,char *where) {
 	strcat(where,tmp);
 };
 
+/* str_replace();
+   params: string, string to search, string to put on the place of second string
+   return: new allocated string
+   action: replace all occurrences of the second parameter in the first paramter by the third parameter
+ */
+
+char *str_replace(const char *str,const char *where,const char *what){
+	char *a=NULL;
+	int len1=strlen(where);
+	int len2=strlen(what);
+	int len3=strlen(str);
+	char *rval=copy_string(str);
+	char *cur=rval;
+	while ((a=strstr(cur,where))){
+		int len=len3-len1+len2;
+		char *old=rval;
+		rval=new char[len+1];
+		int l=a-old;
+		memcpy(rval,old,l);
+		memcpy(rval+l,what,len2);
+		cur=rval+l+len2;
+		memcpy(rval+l+len2,a+len1,len3-l-len1);
+		len3+=len2-len1;
+		rval[len3]=0;
+		delete[]old;
+	};
+	return rval;
+};
+
 /* convert_time();
     params: number of seconds, pointer to buffer
     return: none
     action: convert number of seconds to nice string HH:MM:SS
  */
 
-void convert_time(int what,char *where) {
+void convert_time(int what,char *where,int TIME_FORMAT) {
 	DBC_RETURN_IF_FAIL(where!=NULL);
 	int hours=what/int(3600);
 	int mins=(what%3600)/int(60);
@@ -218,7 +259,7 @@ void convert_time(int what,char *where) {
 		strcat(where,tmp);
 //		sprintf(where,"%i",mins);
 	};
-	if (!CFG.TIME_FORMAT) {
+	if (!TIME_FORMAT) {
 		strcat(where,":");
 		convert_int_to_2(secs,tmp);
 		strcat(where,tmp);
@@ -280,6 +321,23 @@ int convert_from_hex(unsigned char what) {
 	return -1;
 };
 
+int convert_from_hex(char what) {
+	if (what>='0' && what<='9')
+		return(int(what)-'0');
+	if (what>='a' && what<='f')
+		return (10+int(what)-'a');
+	if (what>='A' && what<='F')
+		return (10+int(what)-'A');
+	return -1;
+};
+
+int convert_from_hex(char a,char b) {
+	int aa=convert_from_hex(a);
+	int bb=convert_from_hex(b);
+	if (aa<0 || bb<0) return 1000;
+	return (aa<<4)|bb;
+};
+
 /* convert_to_hex();
     params: char, pointer to buffer
     return: none
@@ -297,6 +355,11 @@ void convert_to_hex(unsigned char what,char *where) {
 	where[1] = _dec_to_hex_[what%16];
 };
 
+void convert_to_hex_s(char what,char *where) {
+	unsigned char b=*((unsigned char*)&what);
+	convert_to_hex(b,where);
+};
+
 /* parse_percents();
     params: string
     return: new allocated string in which all %xx codes from input string
@@ -344,7 +407,7 @@ char *parse_percents(char *what) {
 char *unparse_percents(char *what) {
 	DBC_RETVAL_IF_FAIL(what!=NULL,NULL);
 	const char *true_chars=".-+%";
-	const char *false_chars=":<>~";
+	const char *false_chars=":<>~ ";
 	char *temp=what;
 	int unparsed_len=0;
 
@@ -375,6 +438,47 @@ char *unparse_percents(char *what) {
 	return rvalue;
 };
 
+static const char *_url_true_chars_=".-+&";
+static const char *_url_false_chars_=":<>~% ";
+
+#include <iostream>
+std::string hexed_string(const std::string &str){
+	std::string rval(str);
+	std::string::size_type len=rval.length(),pos=0;
+	char buf[4]={'%',0,0,0};
+	while(pos<len){
+		char r=rval[pos];
+		if ((r<'/' || r>'z' || index(_url_false_chars_,r))
+		    && index(_url_true_chars_,r)==0){
+			convert_to_hex_s(r,buf+1);
+			rval.replace(pos,1,buf);
+			pos+=2;
+			len+=2;
+		};
+		pos++;
+	};
+	return rval;
+};
+
+std::string unhexed_string(const std::string &str){
+	std::string::size_type pos=0;
+	std::string::size_type len=str.length();
+	std::string rval(str);
+	while((pos=rval.find('%',pos))!=std::string::npos){
+		if (len<pos+1) break;
+		if (len>=pos+2){
+			int b=convert_from_hex(rval[pos+1],rval[pos+2]);
+			if (b<256){
+				char buf[2]={b,0};
+				rval.replace(pos,3,buf);
+				len-=2;
+			};
+		};
+		pos++;
+	};
+	return rval;
+};
+
 /* escape_char();
     params: string, char to search, escaped character;
     return: string
@@ -426,6 +530,11 @@ void str_non_print_replace(char *what,char symbol){
 		if (*temp<' ') *temp=(unsigned char )symbol;
 		temp+=1;
 	};
+	while (temp > (unsigned char *)what){
+		temp--;
+		if (*temp==' ') *temp=0;
+		else break;
+	};
 };
 
 /*  del_crlf()
@@ -448,50 +557,56 @@ void del_crlf(char *what) {
     action: fill buffer by formated integer;
  */
 
-void make_number_nice(char *where,fsize_t num) {
+void make_number_nice(char *where,fsize_t num,int NICE_DEC_DIGITALS) {
 	DBC_RETURN_IF_FAIL(where!=NULL);
-	switch (CFG.NICE_DEC_DIGITALS.curent) {
+	switch (NICE_DEC_DIGITALS) {
 		case 1:
 		case 3:{
-				sprintf(where,"%li",num);
+				sprintf(where,"%lli",num);
 				int len=strlen(where);
 				if (len<4) return;
 				for (int i=len-3;i>0;i-=3,len++) {
 					for (int a=len-1;a>=i;a--)
 						where[a+1]=where[a];
-					if (CFG.NICE_DEC_DIGITALS.curent==1) where[i]=' ';
+					if (NICE_DEC_DIGITALS==1) where[i]=' ';
 					else where[i]='\'';
 				};
 				where[len]=0;
 				break;
 			};
 		case 2:	{
-				int megs;
+				fsize_t gigs,megs;
 				megs=num/(1024*1024);
-				int kils=num/1024;
-				if (megs==0 && kils<1000) {
-					int bytes=((num-kils*1024)*10)/1024;
-					if (kils==0 && bytes<1000)
-						sprintf(where,"%li",num);
-					else
-						sprintf(where,"%i.%iK",kils,bytes);
-				} else{
-					int bytes=((num-megs*1024*1024)*10)/(1024*1024);
-					sprintf(where,"%i.%iM",megs,bytes);
+				gigs=megs/1024;
+				if (gigs==0 && megs<1000){
+					fsize_t kils=num/1024;
+					if (megs==0 && kils<1000) {
+						fsize_t bytes=((num-kils*1024)*10)/1024;
+						if (kils==0 && bytes<1000)
+							sprintf(where,"%lli",num);
+						else
+							sprintf(where,"%lli.%lliK",kils,bytes);
+					} else{
+						fsize_t bytes=((num-megs*1024*1024)*10)/(1024*1024);
+						sprintf(where,"%lli.%lliM",megs,bytes);
+					};
+				}else{
+					fsize_t bytes=((num-gigs*1024*1024*1024)*10)/(1024*1024*1024);
+					sprintf(where,"%lli.%lliG",gigs,bytes);
 				};
 				break;
 			};
 		default:
-			sprintf(where,"%li",num);
+			sprintf(where,"%lli",num);
 	};
 };
 
 /* the same as previos but for long
  */
 
-void make_number_nicel(char *where,unsigned long num) {
+void make_number_nicel(char *where,unsigned long num,int NICE_DEC_DIGITALS) {
 	DBC_RETURN_IF_FAIL(where!=NULL);
-	switch (CFG.NICE_DEC_DIGITALS.curent) {
+	switch (NICE_DEC_DIGITALS) {
 		case 1:
 		case 3:{
 				sprintf(where,"%lu",num);
@@ -500,7 +615,7 @@ void make_number_nicel(char *where,unsigned long num) {
 				for (int i=len-3;i>0;i-=3,len++) {
 					for (int a=len-1;a>=i;a--)
 						where[a+1]=where[a];
-					if (CFG.NICE_DEC_DIGITALS.curent==1) where[i]=' ';
+					if (NICE_DEC_DIGITALS==1) where[i]=' ';
 					else where[i]='\'';
 				};
 				where[len]=0;
@@ -705,7 +820,11 @@ int ctime_to_time(char *src) {
 			sscanf_int(tmpdata,&(date.tm_sec));
 		};
 	};
-	return mktime(&date);
+#ifdef HAVE_STRUCT_TM_TM_ZONE
+	return(mktime(&date)+date.tm_gmtoff);
+#else
+	return(mktime(&date));
+#endif
 };
 
 /* check_mask();
@@ -714,11 +833,11 @@ int ctime_to_time(char *src) {
 	    zero if `src' is not satisfied by `mask'
  */
 
-int check_mask(char *src,char *mask) {
+int check_mask(const char *src,const char *mask) {
 	DBC_RETVAL_IF_FAIL(mask!=NULL,0);
 	DBC_RETVAL_IF_FAIL(src!=NULL,0);
-	char *m=mask;
-	char *s=src;
+	const char *m=mask;
+	const char *s=src;
 	while (*m) {
 		if (*m=='*') {
 			if (*s==0) return 0;
@@ -742,11 +861,11 @@ int check_mask(char *src,char *mask) {
    allow ? as any symbol
  */
 
-int check_mask2(char *src,char *mask) {
+int check_mask2(const char *src,const char *mask) {
 	DBC_RETVAL_IF_FAIL(mask!=NULL,0);
 	DBC_RETVAL_IF_FAIL(src!=NULL,0);
-	char *m=mask;
-	char *s=src;
+	const char *m=mask;
+	const char *s=src;
 	while (*m) {
 		switch(*m){
 		case '*':{
@@ -775,11 +894,11 @@ int check_mask2(char *src,char *mask) {
  */
 
 
-int check_mask2_uncase(char *src,char *mask) {
+int check_mask2_uncase(const char *src,const char *mask) {
 	DBC_RETVAL_IF_FAIL(mask!=NULL,0);
 	DBC_RETVAL_IF_FAIL(src!=NULL,0);
-	char *m=mask;
-	char *s=src;
+	const char *m=mask;
+	const char *s=src;
 	while (*m) {
 		switch(*m){
 		case '*':{
@@ -809,17 +928,17 @@ void normalize_path(char *src) {
 	while (*a) {
 		if (*a=='/'){
 			int need_exit=0;
-			while(!need_exit){
-				switch(*(a+1)){
+			do {
+				switch(a[1]){
 				case '/':
 					a+=1;
 					break;
 				case '.':{
-					if (*(a+2)=='/' || *(a+2)==0){
+					if (a[2]=='/' || a[2]==0){
 						a+=2;
 						break;
 					};
-					if (*(a+2)=='.'  && (*(a+3)=='/' || *(a+3)==0)){
+					if (a[2]=='.'  && (a[3]=='/' || a[3]==0)){
 						if (b>src) b-=1;
 						while (b>src && *b!='/') b-=1;
 						a+=3;
@@ -829,7 +948,8 @@ void normalize_path(char *src) {
 				default:
 					need_exit=1;
 				};
-			};
+				if (*a==0) break;
+			}while(!need_exit);
 		};
 		if ((*b=*a)==0) return;
 		b+=1;
@@ -838,6 +958,29 @@ void normalize_path(char *src) {
 	*b=0;
 };
 
+char *normalize_path_full(char *src){
+	DBC_RETVAL_IF_FAIL(src!=NULL,NULL);
+	if (*src=='~'){
+		struct passwd *p;
+		char *cur=index(src,'/');
+		if (src[1]=='/'){
+			p=getpwuid(getuid());
+		}else{
+			if (cur) *cur=0;
+			p=getpwnam(src+1);
+			if (cur) *cur='/';
+		};
+		if (p){
+			char *path=sum_strings(p->pw_dir,cur,NULL);
+			normalize_path(path);
+			return(path);
+		};
+	};
+	char *rval=copy_string(src);
+	normalize_path(rval);
+	return(rval);
+};
+
 /* FIXME: compose_path() should be rewritten!!!
  */
 
@@ -1001,6 +1144,14 @@ int string_ended_uncase(const char *ended, const char *what){
 	return strncasecmp(aa,bb,min);
 };
 
+std::string filename_extension(const std::string &name){
+	std::string::size_type pos=name.rfind('.');
+	if (pos!=std::string::npos){
+		return name.substr(pos+1);
+	};
+	return std::string();
+};
+
 /* primitive file operations
  */
 
@@ -1008,12 +1159,12 @@ int f_wchar(int fd,char c){
 	return(write(fd,&c,sizeof(char)));
 };
 
-int f_wstr(int fd,char *str){
+int f_wstr(int fd,const char *str){
 	DBC_RETVAL_IF_FAIL(str!=NULL,0);
 	return(write(fd,str,strlen(str)));
 };
 
-int f_wstr_lf(int fd,char *str){
+int f_wstr_lf(int fd,const char *str){
 	DBC_RETVAL_IF_FAIL(str!=NULL,0);
 	int a=f_wstr(fd,str);
 	if (a<0) return a;
@@ -1040,14 +1191,14 @@ int f_rstr(int fd,char *where,int max) {
 	return 0;
 };
 
-int write_named_string(int fd,char *name,char *str){
+int write_named_string(int fd,const char *name,const char *str){
 	if (name==NULL || str==NULL) return(0);
 	if (f_wstr_lf(fd,name)<0) return(-1);
 	if (f_wstr_lf(fd,str)<0) return(-1);
 	return 0;
 };
 
-int write_named_integer(int fd,char *name,int num){
+int write_named_integer(int fd,const char *name,int num){
 	if (name==NULL) return(0);
 	if (f_wstr_lf(fd,name)<0) return(-1);
 	char str[MAX_LEN];
@@ -1056,7 +1207,7 @@ int write_named_integer(int fd,char *name,int num){
 	return 0;
 };
 
-int write_named_time(int fd,char *name,time_t when){
+int write_named_time(int fd,const char *name,time_t when){
 	if (name==NULL) return(0);
 	if (f_wstr_lf(fd,name)<0) return(-1);
 	char str[MAX_LEN];
@@ -1065,6 +1216,15 @@ int write_named_time(int fd,char *name,time_t when){
 	return 0;
 };
 
+int write_named_fsize(int fd,const char *name,fsize_t size){
+	if (name==NULL) return(0);
+	if (f_wstr_lf(fd,name)<0) return(-1);
+	char str[MAX_LEN];
+	g_snprintf(str,MAX_LEN,"%lli",size);
+	if (f_wstr_lf(fd,str)<0) return(-1);
+	return 0;
+};
+
 /* length of string with specified integer;
  */
 int int_to_strin_len(int num){
@@ -1118,9 +1278,9 @@ static char* date_fmts[]={
 	"[E]" // extension of a file in upper case
 };
 
-char *get_extension(char *name){
+char *get_extension(const char *name){
 	if (name==NULL) return(NULL);
-	char *a=name+strlen(name)-1;
+	const char *a=name+strlen(name)-1;
 	while(a>=name){
 		if (*a=='.'){
 			a+=1;
@@ -1135,7 +1295,7 @@ char *get_extension(char *name){
 	return(NULL);
 };
 
-char *parse_save_path(const char *str,char *file){
+char *parse_save_path(const char *str,const char *file){
 	DBC_RETVAL_IF_FAIL(str!=NULL,NULL);
 	int len=0;
 	char *rval;
@@ -1272,6 +1432,14 @@ tPStr::tPStr(){
 	a=NULL;
 };
 
+tPStr::tPStr(const char*s){
+	a=copy_string(s);
+};
+
+bool tPStr::notempty(){
+	return(a!=NULL && a[0]!=0);
+};
+
 void tPStr::set(const char *b){
 	if (a) delete[] a;
 	a=copy_string(b);
@@ -1284,6 +1452,22 @@ tPStr::~tPStr(){
 
 /***************************************************************/
 
+void make_proxy_host(const char *host,int port){
+	char *tmp=new char[strlen(host)+10];
+	char *b=tmp;
+	while(*host!=':' && *host){
+		*b=*host;
+		host++;
+		b++;
+	};
+	*b=':';
+	b++;
+	sprintf(b,"%i",port);
+	if (*tmp!=':' && *tmp)
+		ALL_HISTORIES[PROXY_HISTORY]->add(tmp);
+	delete[] tmp;
+};
+
 int file_copy(char *from,char *to,char *buf,int size){
 	FILE *src=fopen(from,"r");
 	if (src==NULL){
