@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -22,22 +22,22 @@ class tMLog:public tStringList{
 	protected:
 	int current_line;
 	time_t start;
-	GtkCList *list;
+	GtkTreeView *list;
 	GtkWidget *open_row_item;
 	GtkWidget *clear_item;
 	void add_to_list();
 	int fd;
 	tStringDialog *string;
-	tAddr *last_error;
-	void real_open_row(int row);
+	d4x::URL last_error;
+	void real_open_row(GtkTreeIter *iter);
 	public:
 		GtkWidget *popup_menu;
 		tMLog();
-		void open_row(int row);
+		void open_row(GtkTreeIter *iter);
 		void open_selected_row();
 		void print();
 		void reinit(int a);
-		void init_list(GtkCList *clist);
+		void init_list(GtkTreeView *clist);
 		void add(char *str,int len,int type);
 		void add(char *str,int type);
 		void add(char *str);
