@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -16,9 +16,28 @@
 
 tMeter::tMeter() {
 	sort=new tSortTree;
+	counter=0;
+	mode=0;
+	lastval=0;
 };
 
-void tMeter::add(int speed) {
+void tMeter::set_mode(int m){
+	mode=m;
+};
+
+void tMeter::add(BSize speed) {
+	lastval=speed;
+	if (mode && counter<10){
+		counter+=1;
+		tSortNode *temp=last();
+		if (temp){
+			sort->del(temp);
+			temp->key+=speed;
+			sort->add(temp);
+			return;
+		};
+	};
+	counter=0;
 	tSortNode *temp=new tSortNode;
 	temp->key=speed;
 	insert(temp);
@@ -42,30 +61,34 @@ tSortNode *tMeter::next() {
 	return (tSortNode *)(tQueue::next());
 };
 
-int tMeter::max() {
+tMeter::BSize tMeter::max() {
 	tSortNode *temp=(tSortNode *)(sort->max());
 	if (temp) return(temp->key);
 	return 0;
 };
 
-int tMeter::first_value() {
+tMeter::BSize tMeter::first_value() {
 	tSortNode *temp=first();
 	if (temp) return temp->key;
 	return 0;
 };
 
-int tMeter::last_value() {
+tMeter::BSize tMeter::last_value() {
 	tSortNode *temp=last();
 	if (temp) return temp->key;
 	return 0;
 };
 
-int tMeter::next_value() {
+tMeter::BSize tMeter::next_value() {
 	tSortNode *temp=next();
 	if (temp)  return temp->key;
 	return 0;
 };
 
+tMeter::BSize tMeter::last_speed(){
+	return(lastval);
+};
+
 tMeter::~tMeter() {
 	delete(sort);
 };
