@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -14,21 +14,27 @@
 #include "queue.h"
 
 class tMeter:public tQueue{
-	protected:
+public:
+	typedef unsigned long long BSize;
+protected:
 	tSortTree *sort;
-	int MAX,NUM;
-	public:
-		tMeter();
-		void add(int speed);
-		void dispose();
-		tSortNode *first();
-		tSortNode *last();
-		tSortNode *next();
-		int max();
-		int first_value();
-		int last_value();
-		int next_value();
-		~tMeter();
+	BSize MAX,NUM;
+	int counter,mode;
+	BSize lastval;
+public:
+	tMeter();
+	void add(BSize speed);
+	void dispose();
+	tSortNode *first();
+	tSortNode *last();
+	tSortNode *next();
+	BSize max();
+	BSize last_speed();
+	BSize first_value();
+	BSize last_value();
+	BSize next_value();
+	void set_mode(int mode);
+	~tMeter();
 };
 
 #endif
