@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -20,7 +20,9 @@ struct tSavedVar{
 
 enum SV_TYPES{
 	SV_TYPE_INT,
+	SV_TYPE_LINT,
 	SV_TYPE_PSTR,
+	SV_TYPE_STDSTR,
 	SV_TYPE_SPLIT,
 	SV_TYPE_CFG,
 	SV_TYPE_URL,
@@ -28,6 +30,11 @@ enum SV_TYPES{
 	SV_TYPE_DOWNLOAD,
 	SV_TYPE_RULE,
 	SV_TYPE_FILTER,
+	SV_TYPE_QDOWNLOAD,
+	SV_TYPE_QUEUE,
+	SV_TYPE_QV,
+	SV_TYPE_ALT,
+	SV_TYPE_TMP,
 	SV_TYPE_END
 };
 
