@@ -1,5 +1,5 @@
 /*	WebDownloader for X-Window
- *	Copyright (C) 1999-2001 Koshelev Maxim
+ *	Copyright (C) 1999-2002 Koshelev Maxim
  *	This Program is free but not GPL!!! You can't modify it
  *	without agreement with author. You can't distribute modified
  *	program but you can distribute unmodified program.
@@ -12,7 +12,9 @@
 #define _D4X_SOUND_HEADER_
 
 #include "queue.h"
+#include "mutex.h"
 #include <pthread.h>
+#include <list>
 
 enum SOUND_EVENTS{
 	SND_STARTUP,
@@ -24,15 +26,20 @@ enum SOUND_EVENTS{
 	SND_LAST
 };
 
-struct d4xSndEvent:public tNode{
-	int event;
-	time_t birth;
-	void print(){};
+namespace d4x{
+
+	struct SndEvent{
+		int event;
+		time_t birth;
+		SndEvent(int _event):event(_event),birth(time(NULL)){};
+		SndEvent(const SndEvent &ev):event(ev.event),birth(ev.birth){};
+	};
+
 };
 
 class d4xSndServer{
-	tQueue *queue;
-	pthread_mutex_t my_mutex,exit_lock;
+	std::list<d4x::SndEvent> queue;
+	d4xMutex my_mutex,exit_lock;
 	pthread_t thread_id;
 	char *snd_table[SND_LAST];
 	void play_sound(int event);
